/* Beispiele aus dem Buch "LDAP fuer Java-Entwickler", 
 * Stefan Zoerner, 4. aktualisierte Auflage 2013
 *  
 * Webseite zum Buch: http://www.ldapjava.de/ 
 * Software & Support Media GmbH / entwickler.press, http://www.entwickler-press.de/
 */
package ldapbuch.kapitel5.ad;

import java.math.BigInteger;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.TimeZone;

/**
 * Toolklasse mit Berechnungsfunktionen f&uuml;r das Active Directory.
 * Enth&auml;lt Hilfsmethoden, um mit speziellen Zeitformaten im AD umzugehen.
 * 
 * @author Stefan Z&ouml;rner
 */
public class AdTools {

  /**
   * Privater Konstruktor, um das Erzeugen von Objekten zu verhindern
   */
  private AdTools() {
  }

  // Konstanten zur Berechnungen
  //
  private static final BigInteger ZEHN_HOCH_SIEBEN = new BigInteger("10000000");

  private static final BigInteger SEKUNDEN_PRO_TAG = new BigInteger("86400");

  /**
   * Rechnet eine Zeitangabe von verstrichenen Einheiten 100ns seit 1601 in ein
   * entsprechendes Java-Calendar-Objekt um. Dabei wird die Standardzeitzone
   * angenommen.
   * 
   * @param hundertNanos
   *          Wert fuer die Zeiteinheiten
   * @throws NumberFormatException
   *           Angabe fuer hundertNanos ist keine Zahl
   * 
   * @return Calender, welche das entsprechende Datum reprasentiert
   */
  public static final Calendar hundertNanosAlsCalendar(String hundertNanos)
      throws NumberFormatException {
    return AdTools.hundertNanosAlsCalendar(TimeZone.getDefault().getID(),
        hundertNanos);
  }

  /**
   * Rechnet eine Zeitangabe von verstrichenen Einheiten 100ns seit 1601 in ein
   * entsprechendes Java-Calendar-Objekt um. In dieser Variante kann auch eine
   * Zeitzone angegeben werden. Das ist z.B. noetig, wenn der Domain-Controller
   * mit der Einstellung GMT arbeitet.
   * 
   * @param timeZone
   *          Zeitzone, z.B. GMT
   * @param hundertNanos
   *          Wert fuer die Zeiteinheiten
   * @throws NumberFormatException
   *           Angabe fuer hundertNanos ist keine Zahl
   * 
   * @return Calender, welche das entsprecehde Datum reprasentiert
   */
  public static final Calendar hundertNanosAlsCalendar(String timeZone,
      String hundertNanos) throws NumberFormatException {

    // Angabe in Tage seit 01.01.1601 umrechnen, die am
    // letzten Tag verstrichenen Sekunden ermitteln
    BigInteger bHundertNanos = new BigInteger(hundertNanos);
    BigInteger bSekunden = bHundertNanos.divide(ZEHN_HOCH_SIEBEN);
    BigInteger bTage = bSekunden.divide(SEKUNDEN_PRO_TAG);
    BigInteger bSekAmTag = bSekunden.mod(SEKUNDEN_PRO_TAG);

    // Calenderobjekt fuer den 01.01.1601
    // in der gewuenschten Zeitzone erstellen
    Calendar cal = new GregorianCalendar(1601, Calendar.JANUARY, 1);
    cal.setTimeZone(TimeZone.getTimeZone(timeZone));

    // verstrichene Tage und Sekunden hinzuziehen
    cal.add(Calendar.DAY_OF_YEAR, bTage.intValue());
    cal.add(Calendar.SECOND, bSekAmTag.intValue());
    return cal;
  }

  /**
   * Rechnet eine Angabe in 100ns-Einheiten in Tage um.
   * 
   * @param hundertNanos
   *          Wert fuer die Zeiteinheiten
   * @return Anzahl Tage (gerundet auf ganze Tage)
   */
  public static final int hundertNanosInTage(String hundertNanos) {
    BigInteger bHundertNanos = new BigInteger(hundertNanos);
    BigInteger bSekunden = bHundertNanos.divide(ZEHN_HOCH_SIEBEN);
    BigInteger bTage = bSekunden.divide(SEKUNDEN_PRO_TAG);
    return bTage.intValue();
  }
}